#!/bin/bash
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Diff two files and PASS if they are equal, FAIL otherwise.
# Usage:
#   diff_test.sh path_to_expected path_to_got

# Find input files
declare -r EXPECTED="$1"
declare -r GOT="$2"

diff_out=$(mktemp /tmp/diff_test.XXXXXXXXX)
diff -cB "$EXPECTED" "$GOT" >"$diff_out"
err=0
if [[ -s "$diff_out" ]] ; then
  cat "$diff_out"
  err=1
  echo 'To update:'
  echo '  cp bazel-bin/'"$GOT" "$EXPECTED"
  echo FAIL
else
  echo PASS
fi
exit $err
